DROP VIEW IF EXISTS auth_user_resource_v;

DROP VIEW IF EXISTS business_role_v;

DROP VIEW IF EXISTS role_membership_v;

CREATE VIEW auth_user_resource_v AS 
SELECT
	br.name AS brole,
	br.unique_id AS unique_business_role_id,
	su.display_name AS user_name,
	su.unique_user_id,
	sp.name AS permission_name,
	sp.unique_permission_id,
	rp.name AS role_name,
	rp.unique_policy_id,
	ds.name AS application,
	ds.unique_id AS unique_application_id,
	aur.start_date,
	aur.end_date,
	aur.in_validity_period,
	aur.in_grace_period,
	aur.indirect_authorization,
	aur.auto_grant,
	aur.auto_revoke,
	aur.create_date,
	aur.deleted,
	aur.delete_date
FROM auth_user_resource aur
	JOIN user_resource ur ON aur.user_resource_id = ur.id
	JOIN business_role br ON aur.unique_business_role_id = br.unique_id AND br.approval_state = 'PUBLISHED'
	JOIN suser su ON su.unique_user_id = ur.unique_user_id AND su.effective = true
	LEFT JOIN spermission sp ON sp.unique_permission_id = ur.unique_resource_id AND sp.effective = true
	LEFT JOIN role_policy rp ON rp.unique_policy_id = ur.unique_resource_id AND rp.deleted = false
	LEFT JOIN data_source ds ON (ds.id::text = ur.unique_resource_id OR ds.id = sp.application) AND ds.deleted = false;

CREATE VIEW business_role_v AS 
 SELECT b_role.id AS business_role_id,
    b_role.unique_id,
    b_role.name,
    b_role.description,
    b_role.grace_period,
    b_role.risk,
    b_role.update_time,
    COALESCE(user_owners.user_owners, ''::text) AS user_owners,
    COALESCE(group_owners.group_owners, ''::text) AS group_owners,
    b_role.approval_state,
    su1.display_name AS created_by,
    su2.display_name AS published_by,
    su3.display_name AS changed_by,
    su4.display_name AS approved_by,
    COALESCE(user_manage.user_managers, ''::text) AS user_managers,
    COALESCE(group_manage.group_managers, ''::text) AS group_managers,
    su5.display_name AS user_fulfiller,
    sgroup.name AS group_fulfiller,
    b_role.approval_note,
    ap.name AS policy_name,
    ap.implicit AS approval_not_required,
    ap.description AS policy_description,
    ap.creation_time AS policy_creation_time,
    ap.update_time AS last_policy_update_time,
    COALESCE(pol_users.policy_users, ''::text) AS policy_users,
    COALESCE(pol_groups.policy_groups, ''::text) AS policy_groups,
    br_crit.criteria AS membership_criteria,
    COALESCE(incl_users.included_users, ''::text) AS included_users,
    COALESCE(incl_groups.included_groups, ''::text) AS included_groups,
    COALESCE(excl_users.excluded_users, ''::text) AS excluded_users,
    COALESCE(excl_groups.excluded_groups, ''::text) AS excluded_groups,
    auth_perm.auth_permission,
    COALESCE(auth_tech_role.auth_tech_role, ''::text) AS auth_tech_role,
    COALESCE(auth_app.auth_application, ''::text) AS auth_application,
    incl_brole.bnames AS included_broles,
    b_role.last_detect_time
   FROM business_role b_role
     LEFT JOIN suser su1 ON b_role.created_by::text = su1.unique_user_id::text AND su1.display_name IS NOT NULL AND su1.deleted = false AND su1.effective = true
     LEFT JOIN suser su2 ON b_role.published_by::text = su2.unique_user_id::text AND su2.display_name IS NOT NULL AND su2.deleted = false AND su2.effective = true
     LEFT JOIN suser su3 ON b_role.changed_by::text = su3.unique_user_id::text AND su3.display_name IS NOT NULL AND su3.deleted = false AND su3.effective = true
     LEFT JOIN suser su4 ON b_role.approved_by::text = su4.unique_user_id::text AND su4.display_name IS NOT NULL AND su4.deleted = false AND su4.effective = true
     JOIN auth_scope ON auth_scope.unique_business_role_id::text = b_role.unique_id::text
     LEFT JOIN auth_role_mapping ar_map ON ar_map.auth_scope = auth_scope.id AND ar_map.auth_role = 29
     LEFT JOIN ( SELECT auth_scope_1.unique_business_role_id,
            string_agg(DISTINCT suser.display_name::text, ' | '::text ORDER BY (suser.display_name::text)) AS user_managers
           FROM auth_role_mapping ar_map_1
             JOIN suser ON ar_map_1.unique_user_id::text = suser.unique_user_id::text AND suser.display_name IS NOT NULL AND suser.deleted = false AND suser.effective = true
             JOIN auth_scope auth_scope_1 ON ar_map_1.auth_scope = auth_scope_1.id
          WHERE ar_map_1.unique_user_id IS NOT NULL AND ar_map_1.auth_role = 27
          GROUP BY auth_scope_1.unique_business_role_id) user_manage ON user_manage.unique_business_role_id::text = b_role.unique_id::text
     LEFT JOIN ( SELECT auth_scope_1.unique_business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS group_managers
           FROM auth_role_mapping ar_map_1
             JOIN sgroup sgroup_1 ON ar_map_1.unique_group_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL
             JOIN auth_scope auth_scope_1 ON ar_map_1.auth_scope = auth_scope_1.id
          WHERE ar_map_1.unique_group_id IS NOT NULL AND ar_map_1.auth_role = 27
          GROUP BY auth_scope_1.unique_business_role_id) group_manage ON group_manage.unique_business_role_id::text = b_role.unique_id::text
     LEFT JOIN suser su5 ON ar_map.unique_user_id::text = su5.unique_user_id::text AND su5.display_name IS NOT NULL AND su5.deleted = false AND su5.effective = true
     LEFT JOIN sgroup ON ar_map.unique_group_id::text = sgroup.unique_group_id::text AND sgroup.name IS NOT NULL AND sgroup.deleted = false AND sgroup.effective = true
     LEFT JOIN approval_policy ap ON
        CASE
            WHEN b_role.approval_policy IS NULL THEN ap.global = true AND ap.deleted = false
            ELSE b_role.approval_policy = ap.id
        END
     LEFT JOIN business_role_memb_crit br_crit ON b_role.id = br_crit.business_role_id
     LEFT JOIN ( SELECT incl_brole2.business_role_id,
            string_agg(incl_brole2.name::text, ' | '::text ORDER BY incl_brole2.business_role_id) AS bnames
           FROM business_role_incl_brole incl_brole2
          GROUP BY incl_brole2.business_role_id) incl_brole ON incl_brole.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_owner_1.business_role_id,
            string_agg(DISTINCT su.display_name::text, ' | '::text ORDER BY (su.display_name::text)) AS user_owners
           FROM business_role_owner br_owner_1
             JOIN suser su ON br_owner_1.unique_owner_id::text = su.unique_user_id::text AND su.display_name IS NOT NULL AND su.deleted = false AND su.effective = true
          GROUP BY br_owner_1.business_role_id) user_owners ON user_owners.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_owner_1.business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS group_owners
           FROM business_role_owner br_owner_1
             JOIN sgroup sgroup_1 ON br_owner_1.unique_owner_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          GROUP BY br_owner_1.business_role_id) group_owners ON group_owners.business_role_id = b_role.id
     LEFT JOIN ( SELECT apmemb.approval_policy_id,
            string_agg(DISTINCT su.display_name::text, ' | '::text ORDER BY (su.display_name::text)) AS policy_users
           FROM approval_policy_memb apmemb
             JOIN suser su ON apmemb.unique_member_id::text = su.unique_user_id::text AND su.display_name IS NOT NULL AND su.deleted = false AND su.effective = true
          WHERE apmemb.is_user = true
          GROUP BY apmemb.approval_policy_id) pol_users ON pol_users.approval_policy_id = b_role.approval_policy
     LEFT JOIN ( SELECT apmemb.approval_policy_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS policy_groups
           FROM approval_policy_memb apmemb
             JOIN sgroup sgroup_1 ON apmemb.unique_member_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          WHERE apmemb.is_user = false
          GROUP BY apmemb.approval_policy_id) pol_groups ON pol_groups.approval_policy_id = b_role.approval_policy
     LEFT JOIN ( SELECT br_incl_mem.business_role_id,
            string_agg(DISTINCT su.display_name::text, ' | '::text ORDER BY (su.display_name::text)) AS included_users
           FROM business_role_incl_memb br_incl_mem
             JOIN suser su ON br_incl_mem.unique_member_id::text = su.unique_user_id::text AND su.display_name IS NOT NULL AND su.deleted = false AND su.effective = true
          WHERE br_incl_mem.is_user = true
          GROUP BY br_incl_mem.business_role_id) incl_users ON incl_users.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_mem.business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS included_groups
           FROM business_role_incl_memb br_incl_mem
             JOIN sgroup sgroup_1 ON br_incl_mem.unique_member_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          WHERE br_incl_mem.is_user = false
          GROUP BY br_incl_mem.business_role_id) incl_groups ON incl_groups.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_excl_mem.business_role_id,
            string_agg(DISTINCT suser.display_name::text, ' | '::text ORDER BY (suser.display_name::text)) AS excluded_users
           FROM business_role_excl_memb br_excl_mem
             JOIN suser ON br_excl_mem.unique_member_id::text = suser.unique_user_id::text AND suser.display_name IS NOT NULL AND suser.deleted = false AND suser.effective = true
          WHERE br_excl_mem.is_user = true
          GROUP BY br_excl_mem.business_role_id) excl_users ON excl_users.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_excl_mem.business_role_id,
            string_agg(DISTINCT sgroup_1.name::text, ' | '::text ORDER BY (sgroup_1.name::text)) AS excluded_groups
           FROM business_role_excl_memb br_excl_mem
             JOIN sgroup sgroup_1 ON br_excl_mem.unique_member_id::text = sgroup_1.unique_group_id::text AND sgroup_1.name IS NOT NULL AND sgroup_1.deleted = false AND sgroup_1.effective = true
          WHERE br_excl_mem.is_user = false
          GROUP BY br_excl_mem.business_role_id) excl_groups ON excl_groups.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_item.business_role_id,
            string_agg(DISTINCT spermission.name::text, ' | '::text ORDER BY (spermission.name::text)) AS auth_permission
           FROM business_role_incl_item br_incl_item
             JOIN spermission ON br_incl_item.unique_item_id::text = spermission.unique_permission_id::text AND spermission.name IS NOT NULL AND spermission.deleted = false AND spermission.effective = true
          WHERE br_incl_item.item_type::text = 'PERMISSION'::text
          GROUP BY br_incl_item.business_role_id) auth_perm ON auth_perm.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_item.business_role_id,
            string_agg(DISTINCT role_policy.name::text, ' | '::text ORDER BY (role_policy.name::text)) AS auth_tech_role
           FROM business_role_incl_item br_incl_item
             JOIN role_policy ON br_incl_item.unique_item_id::text = role_policy.unique_policy_id::text AND role_policy.deleted = false
          WHERE br_incl_item.item_type::text = 'TECHNICAL_ROLE'::text
          GROUP BY br_incl_item.business_role_id) auth_tech_role ON auth_tech_role.business_role_id = b_role.id
     LEFT JOIN ( SELECT br_incl_item.business_role_id,
            string_agg(DISTINCT data_source.name::text, ' | '::text ORDER BY (data_source.name::text)) AS auth_application
           FROM business_role_incl_item br_incl_item
             JOIN data_source ON br_incl_item.unique_item_id::text = data_source.id::text AND data_source.deleted = false
          WHERE br_incl_item.item_type::text = 'APPLICATION'::text
          GROUP BY br_incl_item.business_role_id) auth_app ON auth_app.business_role_id = b_role.id
  WHERE b_role.approval_state::text <> 'ARCHIVE'::text;

CREATE VIEW role_membership_v AS 
 SELECT rp.name AS role_name,
    rp.unique_policy_id,
    rp.state AS role_state,
    rp.deleted AS role_deleted,
    su.display_name AS user_display_name,
    su.unique_user_id,
    ru.suser AS user_id,
    ru.policy AS policy_id,
    ru.detection_time AS role_user_detection_time,
    ru.deleted AS role_user_deleted,
    ru.deletion_time AS role_user_deletion_time
   FROM role_user ru
     JOIN role_policy rp ON ru.policy = rp.id
     JOIN suser su ON ru.suser = su.id;

ALTER TABLE auth_user_resource_v
  OWNER TO igops;
GRANT ALL ON TABLE auth_user_resource_v TO igops;
GRANT SELECT ON TABLE auth_user_resource_v TO ig_report_role;

ALTER TABLE business_role_v
  OWNER TO igops;
GRANT ALL ON TABLE business_role_v TO igops;
GRANT SELECT ON TABLE business_role_v TO ig_report_role;

ALTER TABLE role_membership_v
  OWNER TO igops;
GRANT ALL ON TABLE role_membership_v TO igops;
GRANT SELECT ON TABLE role_membership_v TO ig_report_role;